@@ -229,8 +229,11 @@ class Server(object):
                             else:
                                 r = packets
                         except IOError:
-                            del self.sockets[sid]
+                            if sid in self.sockets:  # pragma: no cover
+                                del self.sockets[sid]
                             r = self._bad_request()
+                        if sid in self.sockets and self.sockets[sid].closed:
+                            del self.sockets[sid]
             elif method == 'POST':
                 if sid is None or sid not in self.sockets:
                     self.logger.warning('Invalid session %s', sid)
@@ -280,7 +283,7 @@ class Server(object):
                      self.async['thread_class'])(target=target, args=args,
                                                  kwargs=kwargs)
         th.start()
-        return th
+        return th  # pragma: no cover
 
     def _generate_id(self):
         """Generate a unique session id."""
@@ -305,8 +308,7 @@ class Server(object):
             return self._unauthorized()
 
         if transport == 'websocket':
-            s.handle_get_request(environ, start_response)
-            return self._ok()
+            return s.handle_get_request(environ, start_response)
         else:
             s.connected = True
             headers = None
